<?php

namespace Service;

use Zeedhi\Framework\Controller\Exception;
use \Util\Utilities;

class Acl {

	protected $entityManager;
	protected $environment;
	protected $profileFactory;
	protected $ruleFactory;
	protected $structureService;
    protected $partnerProfileFactory;
    /**
     * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Profile
     */
    protected $profileRepository;

    const ALLOWED = 'A';
    const READ_ONLY = 'R';
    const INVISIBLE = 'I';

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Util\Utilities $environment, \Factory\Profile $profileFactory,
								\Factory\Rule $ruleFactory, \Service\Structure $structureService, \Factory\PartnerProfile $partnerProfileFactory){
		$this->entityManager = $entityManager;
		$this->environment = $environment;
		$this->profileFactory = $profileFactory;
		$this->ruleFactory = $ruleFactory;
		$this->structureService = $structureService;
		$this->partnerProfileFactory = $partnerProfileFactory;
		$this->profileRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Profile');
	}

	protected function getParentPermission($parent, $rules){
		if($parent === null){
			return self::ALLOWED;
		}
		foreach ($rules as $rule) {
			if($rule['ID'] === $parent){
				if($rule['RULE'] === null){
					$rule['RULE'] = $this->getParentPermission($rule['PARENT'], $rules);
				}
				return $rule['RULE'];
			}
		}
	}

	public function getElementsByProfile($profile){
		$rules = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Element')
			->getElementsByProfile($this->environment->getCurrentOrganizationId(), $profile);
		foreach ($rules as &$rule) {
			if($rule['RULE'] === null){
				$rule['RULE'] = $this->getParentPermission($rule['PARENT'], $rules);
			}
		}
		return $rules;
	}

	public function getProfiles($profile = null){
		if(empty($profile)){
			$structures = $this->structureService->getStructuresOwnedByManagerOnline();
			foreach ($structures as $structureKey => $structureValue) {
				$structures[$structureKey] = $structureValue['ID'];
			}
			$structures = empty($structures) ? '(0)' : implode(', ', $structures);
			return $this->profileRepository->getProfiles($structures,
				$this->environment->getCurrentOrganizationId());
		}
		else{
			return $this->profileRepository->getProfile($profile);
		}
	}
	public function findProfileByExternalId($externalId){

        return $this->profileRepository->findOneBy(array('EXTERNAL_ID' => $externalId));

	}

	public function persistProfile($id, $label, $category, $structure, $control){
		$structure = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure', $structure);
		if(empty($id)){
			$entity = $this->profileFactory->createProfile(Profile::EMPLOYEE_CATEGORY, $label, 'A', $structure, $control);
		}
		else{
			$entity = $this->profileRepository->find( $id);
			$entity = $this->profileFactory->updateProfile($entity, $category, $label, 'A', $structure, $control);
		}
		$this->entityManager->persist($entity);
		$this->entityManager->flush();
        return $entity;
	}

	public function persistRule($id, $element, $profile, $rule){
		if(empty($id)){
			$element = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Element', $element);
			$profile = $this->profileRepository->find( $profile);
			$entity = $this->ruleFactory->createRule($element, $profile, $rule);
		}
		else{
			$entity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Rule', $id);
			$entity = $this->ruleFactory->updateRule($entity, $rule);
		}
		$this->entityManager->persist($entity);
		$this->entityManager->flush();
	}

	public function deleteProfile($id){
		$connection = $this->entityManager->getConnection();
		try{
			$connection->beginTransaction();
			$associations = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\PartnerProfile')->findBy(array('profile' => $id));
			foreach ($associations as $association) {
				$this->entityManager->remove($association);
			}
			$this->entityManager->flush();

			$entity = $this->profileRepository->find($id);
			$this->entityManager->remove($entity);		
			$this->entityManager->flush();
			$connection->commit();
		}
		catch(\Exception $e){
			$connection->rollBack();
			throw $e;
		}
	}

	public function getResourcesByProfile($profile){
		return $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource')
			->getResourcesByProfile($this->environment->getCurrentOrganizationId(), $profile);
	}

	public function persistProfileResource($resource, $profile){
		$connection = $this->entityManager->getConnection();
		try{
			$connection->beginTransaction();
			$resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resource);
			$profile = $this->profileRepository->find($profile);
			$partnerProfile = $this->partnerProfileFactory->createPartnerProfile($resource, $profile);
			$this->entityManager->persist($partnerProfile);
			$this->entityManager->flush();
			$connection->commit();
		}
		catch(\Exception $e){
			$connection->rollBack();
			throw $e;
		}
	}

	public function deleteProfileResource($resource, $profile){
		$connection = $this->entityManager->getConnection();
		try{
			$connection->beginTransaction();
			$partnerProfile = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\PartnerProfile')
				->findOneBy(array('resource' => $resource, 'profile' => $profile));
			$this->entityManager->remove($partnerProfile);
			$this->entityManager->flush();
			$connection->commit();
		}
		catch(\Exception $e){
			$connection->rollBack();
			throw $e;
		}
	}
}