<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;
use Zeedhi\DevOps\Util\Utilities;

class BudgetItem extends EntityRepository {

    public function budgetItem($organizationId,$conditionsSQL,$limit, $offset,$notClassificationId = null) {
        $notClassification = '';
        if($notClassificationId){
            $notClassification = <<<SQL
            AND NOT EXISTS ( SELECT  BUI.ID
                              FROM CLASSIFICATION_BUDGET_ITEM CLB
                             WHERE CLB.BUDGET_ITEM = BUI.ID
                            AND CLB.CLASSIFICATION = $notClassificationId)
SQL;
        }
        $query = <<<SQL
        SELECT * FROM (
			SELECT BUI.ID,BUI.ORGANIZATION_ID ,RES.NAME AS CREATED_BY ,BUI.CREATED_AT ,BUI.UPDATED_BY ,BUI.UPDATED_AT ,BUI.NAME ,
			       TO_CHAR(BUI.VALUE,'999G999G999G990D00', 'NLS_NUMERIC_CHARACTERS = '',.''') AS VALUE_ITEM,
			       BUI.UNIT_MEASURE,BUI.STATUS,
			       (SELECT BIB.ID FROM BUDGET_ITEM_BUDGET BIB
                            WHERE BIB.BUDGET_ITEM = BUI.ID
                              AND  ROWNUM = 1) AS NOT_DELETE
			  FROM BUDGET_ITEM BUI
			  JOIN RESOURCE_T RES ON (BUI.UPDATED_BY = RES.ID)
			 WHERE BUI.ORGANIZATION_ID = :organizationId
			   AND BUI.STATUS <> :statusDeleted
			   $notClassification
			   )
			   WHERE 1 = 1
			    $conditionsSQL
SQL;
        $params = array(
            'organizationId' => $organizationId,
            'statusDeleted' => LogicalDeleted::STATUS_LOGICAL_DELETED,
        );

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findIssueBudget($issueId, $proposalTitle = 'Proposta' ,$discountTitle = 'Desconto',
                                    $namePercent = 'Percentual') {
        $query =
            <<<SQL
             SELECT '  '||'$proposalTitle' AS TYPE_ORDER,
                    TO_CHAR(BUI.ID) AS BUDGET_ITEM_ID,
                    BUI.NAME AS ITEM_NAME,
                    NVL(null, 0) AS PROPOSAL_QUANTITY,
                    TO_CHAR(NVL(BUI.VALUE, 0),'999G999G999G990D00', 'NLS_NUMERIC_CHARACTERS = '',.''') AS ITEM_VALUE,
                    NVL(BUI.VALUE, 0) AS ITEM_VALUE_TRUE,
                    NVL(BUI.VALUE, 0) * NVL(null, 0) AS PROPOSAL_VALUE,
                    BUI.UNIT_MEASURE,
                    ISS.ID AS ISSUE_ID
               FROM TASK ISS
               JOIN CLASSIFICATION_BUDGET_ITEM CBI ON (CBI.CLASSIFICATION   = ISS.CLASSIFICATION )
               JOIN BUDGET_ITEM BUI ON (BUI.ID = CBI.BUDGET_ITEM AND BUI.STATUS = 'A')
              WHERE ISS.ID = :ISSUE_ID

              UNION

            SELECT  ' '||'$discountTitle' AS TYPE_ORDER,
                    'XXXX' AS BUDGET_ITEM_ID,
                    ' ' AS ITEM_NAME,
                    NVL(null, 0) AS PROPOSAL_QUANTITY,
                    TO_CHAR(' ') AS ITEM_VALUE,
                    NVL(BUD.DISCOUNT_PERCENT, 0) AS ITEM_VALUE_TRUE,
                    NVL(-BUD.DISCOUNT_VALUE, 0) AS PROPOSAL_VALUE,
                    '$namePercent' AS UNIT_MEASURE,
                    ISS.ID AS ISSUE_ID
               FROM TASK ISS
          LEFT JOIN BUDGET BUD ON (BUD.TASK   = ISS.ID )
              WHERE ISS.ID = :ISSUE_ID
           ORDER BY BUDGET_ITEM_ID

SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

}